/**
 * @file
 * @brief Autogenerated configuration information - do not edit.
 */
#ifndef INC_SPF_VERSION
#define INC_SPF_VERSION

#define SPF_LIB_VERSION_MAJOR	1
#define SPF_LIB_VERSION_MINOR	2
#define SPF_LIB_VERSION_PATCH	11

#endif
